// fileManager.h
#pragma warning (disable:4183)

// Definition of some basic types
// Useful, because sometime we need to be sure of the size of an integer type
// May need to be "#ifdef"ed to fit a particular OS on a particular processor
typedef   signed char  int8;
typedef   signed short int16;
typedef   signed int   int32;
typedef unsigned char  uint8;
typedef unsigned short uint16;
typedef unsigned int   uint32;

// Command
typedef enum
{
    CMD_LIST,
    CMD_EXTRACT
} Command_t;

// RFF header
typedef struct
{
    uint8  Magic [4];
    uint32 Version;
    uint32 DirOffset;
    uint32 DirNbEntries;
    uint32 Unknown1;
    uint32 Unknown2;
    uint32 Unknown3;
    uint32 Unknown4;
} RffHeader_t;

// Directory entry for a file
typedef struct
{
    uint8  Unknown0 [16];
    uint32 Offset;
    uint32 Size;
    uint32 Unknown1;
    uint32 Time;           // Obtained with the "time" standard function
    uint8  Flags;
    char   Name [11];
    uint32 Unknown2;       // ID ? Maybe for an enumeration function...
} DirectoryEntry_t;

// Informations about a packed file
typedef struct
{
    char   Name [13];
    uint8  Flags;
    uint32 Time;
    uint32 Size;
    uint32 Offset;
} FileInfos_t;

// After we read the archive fill in this buffer
struct data_source_t
{
	int				numoffiles;
	vector<int>		size;
	vector<void *>	filebuffer;
	vector<string>  filenames;
};


// Default size of Buffer in Kb
#define DEFAULT_BUFFER_SIZE 4

// File flags
enum
{
    FLAG_NONE      = 0,
    FLAG_ENCRYPTED = (1 << 4)  // 0x10
};

class fileManager_t
{
	public:
		fileManager_t::FileManager_t();
		void Init_Rff(char *filename);
		void Precache(char *name);
		void decrypt_Map(char *board);

		string fileAddPath(string path, string in)
		{
			string fixStr;

			fixStr.append(".\\");
			fixStr.append(path);
			fixStr.append("\\");
			fixStr.append(in);

			return fixStr;
		}
	private:
		data_source_t *cachedFiles;
};