//-------------------------------------------------------------------------
/*
Copyright (C) 1997, 2005 - 3D Realms Entertainment

This file is part of Shadow Warrior version 1.2

Shadow Warrior is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Original Source: 1997 - Frank Maddin and Jim Norwood
Prepared for public release: 03/28/2005 - Charlie Wiederhold, 3D Realms
Modified for WinBlood by Justin Marshall
*/
//-------------------------------------------------------------------------

// game_FAFTest.cpp

#include "game.h"

#define SOSUP1     2332
#define SOSDOWN1   2331
/*
===================================================================================
boolean Game::FindFloorView(short match, LONGp x, LONGp y, LONG z, SHORTp sectnum)
===================================================================================
*/

boolean Game::FindFloorView(short match, LONGp x, LONGp y, LONG z, SHORTp sectnum)
{
	long xoff;
    long yoff;
    short i, nexti;
    spritetype  *thisSprite;
	xspritetype *xsprite;
    short top_sprite = -1;
    long pix_diff;
    long newz;

	// Search Stat List For closest ceiling view sprite
    // Get the match, xoff, yoff from this point
	for(int i = 0; i < this->numofsprites; i++)
	{
		xsprite    = build_func->GetXSprite(i);
		thisSprite = build_func->Game_GetSprite(i);

		if(thisSprite->picnum == SOSDOWN1 && xsprite->data1 == match)
		{
			xoff = *x - thisSprite->x;
            yoff = *y - thisSprite->y;
			break;
		}
	}

	// determine x,y position
	for(int i = 0; i < this->numofsprites; i++)
	{
		xsprite    = build_func->GetXSprite(i);
		thisSprite = build_func->Game_GetSprite(i);

		if(thisSprite->picnum == SOSUP1 && xsprite->data1 == match)
		{
			short upper=0, lower=0;
			long  x2 = thisSprite->x;
			long  y2 = thisSprite->y;

            *x = x2 + xoff;
            *y = y2 + yoff;

            // get new sector
            GetUpperLowerSector(match, *x, *y, &upper, &lower);
            *sectnum = lower;
			break;
		}
	}
	if(*sectnum < 0)
		return FALSE;
	
	sectortype  *thisSector = build_func->GetSector(*sectnum);

	long zval1;

	zval1  = build_func->GetSector(thisSprite->sectnum)->ceilingz;


	// move ceiling multiple of 128 so that the wall tile will line up
    pix_diff = klabs(z - zval1) >> 8;
    newz     = zval1 - ((pix_diff / 128) + 1) * Z(128);


	for(int i = 0; i < this->numofsprites; i++)
	{
		xsprite    = build_func->GetXSprite(i);
		thisSprite = build_func->Game_GetSprite(i);

		if(thisSprite->picnum == SOSDOWN1 && xsprite->data1 == match)
		{
		    tsectortype tempsector;

			tempsector.pic     = thisSector->ceilingpicnum;
			tempsector.sectnum = thisSprite->sectnum;
			tempsector.slope   = thisSector->ceilingheinum;
			tempsector.zval    = thisSector->ceilingz;

			saveSectors.push_back(tempsector);

			thisSector->ceilingz = newz;
			DeletePic(thisSprite->sectnum,0);
			thisSector->ceilingheinum = 0;
		}
	}

	return TRUE;
}

/*
===========================================================================
short Game::ViewSectorInScene(short cursectnum, short type, short type2)
===========================================================================
*/

short Game::ViewSectorInScene(short cursectnum, short type, short type2)
{
	spritetype    *thisSprite,*thisSprite2;
	xspritetype   *xsprite,*xsprite2;
	for(int i = 0; i < numofsprites; i++)
	{
		thisSprite = build_func->Game_GetSprite(i);
		xsprite    = build_func->GetXSprite(i);

		if(thisSprite->picnum == type)
		{
			for(int j = 0; j < numofsprites; j++)
			{
				thisSprite2 = build_func->Game_GetSprite(j);
				xsprite2    = build_func->GetXSprite(j);
				if(xsprite->data1 == xsprite2->data1)
				{
					if(thisSprite2->picnum == type2)
					{	
						if(common->spriteSeePlayer(i))
							return xsprite2->data1;
						/*
						// This needs to be added
						if(common->PicInView(thisSprite->picnum, TRUE))
						{
							if(xsprite->data1 == 10)
								exit(0);
							return xsprite->data1;
						}
						*/
					}
				}
			}
		}
	}
	return -1;
}

/*
==================================================
void Game::DeletePic(short sectnum, short floor)
==================================================
*/

void Game::DeletePic(short sectnum, short floor)
{
	sectortype *thisSector = build_func->GetSector(sectnum);
	if(floor == 0)
		thisSector->floorpicnum   = 4054;
	else
		thisSector->ceilingpicnum = 4054;
}

/*
========================================================================================
void Game::GetUpperLowerSector(short match, long x, long y, short *upper, short *lower)
========================================================================================
*/

void Game::GetUpperLowerSector(short match, long x, long y, short *upper, short *lower)
{
    short *sectorlist = new short[16];
    short sln = 0;
    short SpriteNum, Next;
    spritetype  *thisSprite;
	xspritetype *xsprite;

	for(int i = 0; i < this->numSectors; i++)
	{
		if (build_func->inside(x, y, (short) i) == 1)
		{
			boolean found=false;
			for(int j = 0; j < this->numofsprites; j++)
			{
				thisSprite = build_func->Game_GetSprite(j);
				xsprite    = build_func->GetXSprite(j);
				if(thisSprite->picnum == SOSUP1 && xsprite->data1 == match)
				{
					found = true;
				}
			}
			if (!found)
                 continue;

            sectorlist[sln] = i;
			sln++;
		}
	}

	// might not find ANYTHING if not tagged right    
  //  if (sln == 0)
//        exit(0);

	if (build_func->GetSector(sectorlist[0])->floorz < build_func->GetSector(sectorlist[1])->floorz)
	{
		// swap
        // make sectorlist[0] the LOW sector
        short hold;

        hold = sectorlist[0];
        sectorlist[0] = sectorlist[1];
        sectorlist[1] = hold;
	}
    *lower = sectorlist[0];
    *upper = sectorlist[1];
}

/*
==================================================================
void Game::updatesectorz(long x, long y, long z, short *sectnum)
==================================================================
*/

void Game::updatesectorz(long x, long y, long z, short *sectnum)
{

}

/*
======================================================================================
boolean Game::FindCeilingView(short match, LONGp x, LONGp y, LONG z, SHORTp sectnum)
======================================================================================
*/

boolean Game::FindCeilingView(short match, LONGp x, LONGp y, LONG z, SHORTp sectnum)
{
	long xoff;
    long yoff;
    short i, nexti;
    spritetype  *thisSprite;
	xspritetype *xsprite;
    short top_sprite = -1;
    long pix_diff;
    long newz;

	// Search Stat List For closest ceiling view sprite
    // Get the match, xoff, yoff from this point
	for(int i = 0; i < this->numofsprites; i++)
	{
		xsprite    = build_func->GetXSprite(i);
		thisSprite = build_func->Game_GetSprite(i);

		if(thisSprite->picnum == SOSUP1 && xsprite->data1 == match)
		{
			xoff = *x - thisSprite->x;
            yoff = *y - thisSprite->y;
			break;
		}
	}

	// determine x,y position
	for(int i = 0; i < this->numofsprites; i++)
	{
		xsprite    = build_func->GetXSprite(i);
		thisSprite = build_func->Game_GetSprite(i);

		if(thisSprite->picnum == SOSDOWN1 && xsprite->data1 == match)
		{
			short upper=0, lower=0;
			long  x2 = thisSprite->x;
			long  y2 = thisSprite->y;

            *x = x2 + xoff;
            *y = y2 + yoff;

            // get new sector
            GetUpperLowerSector(match, *x, *y, &upper, &lower);
            *sectnum = upper;
			break;
		}
	}

	if(*sectnum < 0)
		return FALSE;

	sectortype  *thisSector = build_func->GetSector(*sectnum);
	long zval1;

	zval1  = build_func->GetSector(thisSprite->sectnum)->floorz;

	// move ceiling multiple of 128 so that the wall tile will line up
    pix_diff = klabs(z - zval1) >> 8;
    newz     = zval1 - ((pix_diff / 128) + 1) * Z(128);
    
	for(int i = 0; i < this->numofsprites; i++)
	{
		xsprite    = build_func->GetXSprite(i);
		thisSprite = build_func->Game_GetSprite(i);

		if(thisSprite->picnum == SOSUP1 && xsprite->data1 == match)
		{
		    tsectortype tempsector;

			tempsector.pic     = thisSector->floorpicnum;
			tempsector.sectnum = thisSprite->sectnum;
			tempsector.slope   = thisSector->floorheinum;
			tempsector.zval    = thisSector->floorz;

			saveSectors.push_back(tempsector);

			thisSector->floorz = newz;
			DeletePic(thisSprite->sectnum,0);
			thisSector->floorheinum = 0;
		}
	}

	
	return TRUE;
}

/*
======================================
void Game::DeletePicALL(long picnum)
======================================
*/

void Game::DeletePicALL(long picnum)
{
	sectortype *thissector;

	for(int i = 0; i < this->numSectors; i++)
	{
		thissector = build_func->GetSector(i);

		if(thissector->floorpicnum == picnum)
			thissector->floorpicnum = 4750;

		if(thissector->ceilingpicnum == picnum)
			thissector->ceilingpicnum = 4750;
	}
}

/*
===============================
void Game::PushSectors(void)
===============================
*/

void Game::PushSectors(void)
{
	cachedCeilingZ.clear();
	cachedFloorZ.clear();

	for(int i = 0; i < this->numSectors; i++)
	{
		cachedCeilingZ.push_back(build_func->GetSector(i)->ceilingz);
		cachedFloorZ.push_back(build_func->GetSector(i)->floorz);
	}
}

/*
=============================
void Game::PopSectors(void)
=============================
*/

void Game::PopSectors(void)
{
	for(int i = 0; i < this->numSectors; i++)
	{
		build_func->GetSector(i)->ceilingz = cachedCeilingZ.at(i);
		build_func->GetSector(i)->floorz   = cachedFloorZ.at(i);
	}
}

/*
=======================================================================================
void Game::FAFDraw(long tx, long ty, long tz, short tang, long thoriz, short tsectnum)
=======================================================================================
*/

void Game::FAFDraw(long tx, long ty, long tz, short tang, long thoriz, short tsectnum)
{
	short match;
	sectortype *thisSector;

	DeletePicALL(504);
	// Lookng Down
	match = ViewSectorInScene(tsectnum,SOSDOWN1,SOSUP1);
	if(match != -1)
	{
		PushSectors();
		FindCeilingView(match,&tx,&ty,tz,&tsectnum);

		if(tsectnum >= 0)
		{
			build_func->displayLevel(tx,ty,tz,tang,tsectnum,65536);
		
			// reset Z's
			for (int i = 0; i < saveSectors.size(); i++)
			{
				thisSector = build_func->GetSector(saveSectors.at(i).sectnum);
				thisSector->floorz      = saveSectors.at(i).zval;
				//thisSector->floorpicnum = saveSectors.at(i).pic;
				thisSector->floorheinum = saveSectors.at(i).slope;
			}
		}
		saveSectors.clear();
		PopSectors();
	}
	else
	{
		// Looking up
		match = ViewSectorInScene(tsectnum,SOSUP1,SOSDOWN1);
		if(match != -1)
		{
			PushSectors();
				FindFloorView(match,&tx,&ty,tz,&tsectnum);
				if(tsectnum >= 0)
				{
					build_func->displayLevel(tx,ty,tz,tang,tsectnum,65536);
				
					// reset Z's
					for (int i = 0; i < saveSectors.size(); i++)
					{
						thisSector = build_func->GetSector(saveSectors.at(i).sectnum);
						thisSector->ceilingz      = saveSectors.at(i).zval;
						//thisSector->ceilingpicnum = saveSectors.at(i).pic;
						thisSector->ceilingheinum = saveSectors.at(i).slope;
					}	
				}
			PopSectors();
		}
	}
	saveSectors.clear();
}